﻿using System;
using System.Reflection;

namespace Logic
{
  internal static class EmbeddedResourceLoader
  {
    private static bool mResolutionInstalled = false;

    /// <summary>
    /// If it isn't already installed, install an event handler that will guide the runtime to DLLs embedded in this assembly.
    /// </summary>
    public static void InstallAssemblyResolution()
    {
      if ( mResolutionInstalled )
        return;

      AppDomain.CurrentDomain.AssemblyResolve += new ResolveEventHandler( GetEmbeddedAssembly );
      mResolutionInstalled = true;
    }

    private static Assembly GetEmbeddedAssembly( object aSender, ResolveEventArgs aArgs )
    {
      string lResourceName = string.Format(
        "Logic.{0}.dll",
        ( new AssemblyName( aArgs.Name ) ).Name );

      using ( var lStream = Assembly.GetAssembly(typeof(Program)).GetManifestResourceStream( lResourceName ) )
      {
        if ( lStream == null )
          return null;

        Byte[] lAssemblyData = new Byte[ lStream.Length ];

        lStream.Read( lAssemblyData, 0, lAssemblyData.Length );

        return Assembly.Load( lAssemblyData );
      }
    }
  }
}
