using System.Collections.Generic;

namespace Logic
{
	/// <summary>
	/// a univeral quantification
	/// </summary>
	internal class ForAll : Proposition
	{
		private Variable variable;
		private Proposition innerProposition;
		
		internal ForAll( Variable variable,  Proposition innerProposition )
		{
			this.variable = variable;
			this.innerProposition = innerProposition;
		}
		
		internal Variable Variable
		{
			get { return variable; }
		}
		
		internal override bool TrueForUniverse( uint possibleUniverse )
		{
			int numberOfPredicateCombinations = Factory.NumberOfPredicateCombinations;

			for ( int predicateCombination = 0; predicateCombination < numberOfPredicateCombinations; predicateCombination++ )
			{
				if ( UniverseContainsPredicateCombination( possibleUniverse, predicateCombination ) )
				{
					Variable.Instantiate( predicateCombination );
					if ( !innerProposition.TrueForUniverse( possibleUniverse ) )
						return false;
				}
			}
			
			return true;
		}
		
		private static bool UniverseContainsPredicateCombination( uint universe, int predicateCombination )
		{
			return ( ( universe >> predicateCombination ) & 1U ) == 1U;
		}
		
		public override string ToString()
		{
			return string.Format( "({0}){1}", variable, innerProposition );
		}
		
    public override IEnumerable<Predicate> FindPredicates()
    {
      return innerProposition.FindPredicates();
    }
	}
}
