﻿using System.Collections.Generic;
using System.Linq;

namespace Logic
{
  /// <summary>
  /// Description of PredicateDictionary.
  /// </summary>
  internal class Predicates
  {
    private Predicate[] mPredicates;
    
    public Predicates( IEnumerable<Predicate> aPredicates )
    {
      mPredicates = aPredicates.ToArray();
      
      if ( mPredicates.Length > 5 )
        throw new EngineException( "Too many predicates!" );
    }
    
    public IEnumerable<uint> PossibleUniverses
    {
      get
      {
        uint lMaximumUniverseNumber = MaximumUniverseNumber;

        for ( uint i = 1; i <= lMaximumUniverseNumber; i++ )
        {
          yield return i;
        }
      }
    }
    
    public IEnumerable<string> GetPredicateCombinationsInUniverse( uint aPossibleUniverse )
    {
		  int lNumberOfCombinationsOfPredicates = NumberOfCombinationsOfPredicates;
		  
		  for ( int lPredicateCombination = 0; lPredicateCombination < lNumberOfCombinationsOfPredicates; lPredicateCombination++ )
			{
				if ( UniverseContainsPredicateCombination( aPossibleUniverse, lPredicateCombination ) )
				  yield return BuildPredicateCombination( lPredicateCombination );
			}
		}
    
    private string BuildPredicateCombination( int aPredicateCombination )
    {
      string lPredicateCombination = "";
      
      foreach ( Predicate lPredicate in mPredicates )
      {
        if ( ( aPredicateCombination & 1 ) != 0 )
          lPredicateCombination += lPredicate.ToString();
        
        aPredicateCombination >>= 1;
      }
      
      return lPredicateCombination;
    }
    
    private static bool UniverseContainsPredicateCombination( uint aPossibleUniverse, int aPredicateCombination )
		{
			return ( ( aPossibleUniverse >> aPredicateCombination ) & 1U ) == 1U;
		}
    
    internal uint MaximumUniverseNumber
    {
      get { return NumberOfPredicates == 5 ? 0xFFFFFFFF : ( 1U << NumberOfCombinationsOfPredicates ) - 1U; }
    }
    
    private int NumberOfPredicates
    {
      get { return mPredicates.Length; }
    }
    
    private int NumberOfCombinationsOfPredicates
    {
      get { return 1 << NumberOfPredicates; }
    }
    
    public Predicate this[ int aIndex ]
    {
      get { return mPredicates[ aIndex ]; }
    }
  }
}
