using System.Linq;

namespace Logic
{
	/// <summary>
	/// a universal quantification
	/// </summary>
	internal class UniversalGeneralization : UnaryOperator
	{
		private Variable mVariable;
		
		internal UniversalGeneralization( Variable aVariable,  Matrix aInnerProposition )
		  : base( aInnerProposition )
		{
			mVariable = aVariable;
		}
		
		internal Variable Variable
		{
			get { return mVariable; }
		}
		
		internal override bool TrueForUniverse( uint aPossibleUniverse, Predicates aPredicates )
		{
		  return aPredicates
        .GetPredicateCombinationsInUniverse( aPossibleUniverse )
        .All( fPredicateCombination => TrueInInstance( fPredicateCombination, aPossibleUniverse, aPredicates ) );
		}
		
		private bool TrueInInstance( string aPredicateCombination, uint aPossibleUniverse, Predicates aPredicates )
		{
		  Variable.Instantiate( aPredicateCombination );
		  
		  return mInnerProposition.TrueForUniverse( aPossibleUniverse, aPredicates );
		}
		
		public override string ToString()
		{
			return string.Format( "{0},{1}", mVariable, mInnerProposition );
		}
	}
}
